import os
import json
import pkg_resources
from shlex import split, quote
from subprocess import check_output, CalledProcessError
from shutil import copyfile


class GoogleCloudInteractor():
    def __init__(self, *args, **kwargs):
        self.cloud_functions_root_dir = "cloud_functions"

    def compile_script_for_cloud_functions(self,
                                           cloud_function_name: str,
                                           interactor_script: str,
                                           table_name: str,
                                           data_collector_method: str,
                                           data_collector_class: str = None):
        interactor_script_name = interactor_script.replace(".py", "")
        if data_collector_class is None:
            data_collector_class = interactor_script_name
        local_function_dir = os.path.join(self.cloud_functions_root_dir,
                                          cloud_function_name)
        file_main = os.path.join(local_function_dir, "main.py")
        file_requirements = os.path.join(local_function_dir,
                                         "requirements.txt")
        # Create a directory for the files
        os.makedirs(local_function_dir, exist_ok=True)
        # Copy config.yaml, models.py, requirements.txt, interactor script into dir
        for script in [
                "config.yaml", "models.py", "requirements.txt",
                interactor_script
        ]:
            copyfile(script, os.path.join(local_function_dir, script))
        # Create a main.py file
        with open(file_main, "w") as f:
            f.write(
                f"from {interactor_script_name} import {data_collector_class}, models\n\n\n"
            )
            f.write(f"def {cloud_function_name}(request):\n\t")
            f.write(f"interactor = {data_collector_class}(driver_path='')\n\t")
            f.write(
                f"interactor.data_to_sql(data_collector=interactor.{data_collector_method}, con=models.engine, schema=interactor.schema, if_exists='append')\n"
            )

    def deploy_to_cloud_functions(self,
                                  project_id: str,
                                  cloud_function_name: str,
                                  region: str,
                                  timeout: int = 240,
                                  runtime: str = 'python37',
                                  source: str = None):
        """Deploy to cloud functions
        Read docs here ,https://googleapis.dev/python/cloudfunctions/latest/functions_v1/cloud_functions_service.html>
        Read source here <https://github.com/googleapis/python-functions/>
        Parameters
        ----------
        timeout : int
            An integer defining the number of seconds from the initialisation
            of the cloud function after which it will timeout
        source : str
            Source location of the function directory
        """
        # Check that the current project is the one you want to use
        current_project = self.gcloud_config_get_value_project()
        if current_project != project_id:
            self.gcloud_config_set_project(project_id)
        # Default source location is the location the method
        # compile_script_for_cloud_functions saves to
        if source is None:
            source = f"cloud_functions/{cloud_function_name}"
        # Deploy to cloud functions
        self.call_subprocess(
            f"gcloud beta functions deploy {cloud_function_name} --region={region} --timeout={timeout} --source={source} --runtime {runtime} --trigger-http"
        )

    def compile_script_for_cloud_run(self, project_id: str, dockerfile_name: str):
        """Compile an interactor for google cloud run
        """
        # Check that the current project is the one you want to use
        current_project = self.gcloud_config_get_value_project()
        if current_project != project_id:
            self.gcloud_config_set_project(project_id)
        # Copy the referenced dockerfile
        dockerfile = pkg_resources.resource_filename(
            'eugenium',
            f'cloud_run_dockerfiles/{dockerfile_name}.Dockerfile',
        )

    def schedule_cloud_function(self, cloud_function_name: str, schedule: str,
                                region: str, project_id: str):
        """Schedule a cloud function
        Read docs here <https://googleapis.dev/python/cloudscheduler/latest/scheduler_v1/cloud_scheduler.html>
        """
        # Schedule a cloud function
        self.call_subprocess(
            f"gcloud beta scheduler jobs create http {cloud_function_name} --schedule='{schedule}' --uri=https://{region}-{project_id}.cloudfunctions.net/{cloud_function_name}"
        )

    def call_subprocess(self, command, split_command=True, shell=True):
        if split_command:
            command = split(command)
        try:
            output = check_output(command, shell=shell).strip()
        except CalledProcessError as e:
            raise RuntimeError(
                f"command '{quote(e.cmd)}' return with error (code {e.returncode}): {e.output}"
            )
        return output

    def gcloud_config_get_value_project(self):
        current_project = self.call_subprocess(
            "gcloud config get-value project")
        return current_project

    def gcloud_config_set_project(self, project_id):
        self.call_subprocess(f"gcloud config set project {project_id}")



#!/usr/bin/env python

from distutils.core import setup

setup(
    name='Eugenium',
    packages=['eugenium'],
    version='0.0.3',
    license='MIT',
    description=
    'Classes to create and deploy selenium processes on google cloud platform',
    author='Eugene Brown',
    author_email='efbbrown@gmail.com',
    url='https://github.com/efbbrown/eugenium',
    download_url="https://github.com/efbbrown/eugenium/archive/v.0.0.3-alpha.tar.gz",
    install_requires=[
        "selenium",
        "sqlalchemy"
    ])


import logging
from typing import Callable

import sqlalchemy

from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions
from selenium.common.exceptions import NoSuchElementException
from selenium.common.exceptions import StaleElementReferenceException


class BaseWebInteractor():
    """
    """
    def __init__(
            self,
            headless: bool = True,
            driver_path: str = None,
            wait_time: int = 5,
            logging_level: int = 20,  # This is equal to logging.INFO
            *args,
            **kwargs):
        """
            Parameters
            ----------
            headless : bool
                Whether to run selenium in headless mode or not
            driver_path : str
                The location of your chrome driver
            wait_time : int
                The time in seconds for selenium to wait for elements before timing out
                See more here <https://www.selenium.dev/documentation/en/webdriver/waits/>

            Returns
            -------
            None
        """
        chrome_options = webdriver.ChromeOptions()
        self.headless = headless
        if self.headless:
            chrome_options.add_argument("--headless")
            chrome_options.add_argument("--disable-gpu")
            chrome_options.add_argument("window-size=1024,768")
            chrome_options.add_argument("--no-sandbox")
        else:
            self.driver_path = driver_path
        # TODO: Remove the next line if unnecessary
        # Added due to this issue <https://stackoverflow.com/a/65497385/4260991>
        chrome_options.add_experimental_option('excludeSwitches', ['enable-logging'])
        self.driver = webdriver.Chrome(chrome_options=chrome_options)
        self.wait_time = wait_time
        self.ignored_exceptions = (
            NoSuchElementException,
            StaleElementReferenceException,
        )
        logging.basicConfig(level=logging_level)

    # def create_driver(self, driver_path: str = None):
    #     """Creates a driver instance for your interactor

    #     Parameters
    #     ----------
    #     wait_time : int
    #         The time in seconds for selenium to wait for elements before timing out
    #         See more here <https://www.selenium.dev/documentation/en/webdriver/waits/>
    #     driver_path : str
    #         The location of your chrome driver

    #     Returns
    #     -------
    #     None
    #     """
    #     if self.driver_path is None:
    #         self.driver_path = driver_path
    #     return webdriver.Chrome(self.driver_path)

    def find(self, by: str, identifier: str):
        """
        Parameters
        ----------
        by : str
            The method to use in the identifier
            See acceptable values here <https://www.selenium.dev/selenium/docs/api/py/webdriver/selenium.webdriver.common.by.html>
        """
        result = WebDriverWait(
            self.driver,
            self.wait_time,
            ignored_exceptions=self.ignored_exceptions).until(
                expected_conditions.presence_of_element_located(
                    (by, identifier)))
        return result

    def find_multiple(self, by: str, identifier: str):
        result = WebDriverWait(
            self.driver,
            self.wait_time,
            ignored_exceptions=self.ignored_exceptions).until(
                expected_conditions.presence_of_all_elements_located(
                    (by, identifier)))
        return result

    def data_to_sql(self,
                    data_collector: Callable,
                    name: str,
                    con,
                    schema=None,
                    if_exists: str = "fail",
                    **kwargs):
        """Runs a function that outputs a pandas dataframe and commits it to a
        database.

        Parameters
        ----------
        data_collector : Callable
            A function that will run some selenium code and return a pandas
            dataframe.
        name : str
            Name of SQL table.
            See https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.DataFrame.to_sql.html
        con : sqlalchemy.engine.(Engine or Connection) or sqlite3.Connection
            Using SQLAlchemy makes it possible to use any DB supported by that library.
            See https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.DataFrame.to_sql.html
        schema : str, optional
            Specify the schema (if database flavor supports this). If None, use default schema.
            See https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.DataFrame.to_sql.html
        if_exists : {‘fail’, ‘replace’, ‘append’}, default ‘fail’
            How to behave if the table already exists.
                fail: Raise a ValueError.
                replace: Drop the table before inserting new values.
                append: Insert new values to the existing table.
            See https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.DataFrame.to_sql.html

        Returns
        -------
        None

        """
        # try:
        df = data_collector(**kwargs)
        df.to_sql(name=name,
                  con=con,
                  schema=schema,
                  if_exists=if_exists,
                  index=False)
        # except e:
        #     logging.debug(e)

    # def db_create(base, database_uri):
    #     """Creates the tables in a database schema

    #     Parameters
    #     ----------
    #     base : sqlalchemy.ext.declarative.declarative_base
    #         A sqlalchemy Base object with classes defining the tables in your schema
    #     database_uri : str
    #         The address of your database
    #     """
    #     engine = sqlalchemy.create_engine(database_uri)
    #     base.metadata.create_all(engine)
